#version 130

// This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.

#ifdef GLSLANG
#extension GL_GOOGLE_include_directive : enable
#endif

#include "settings.glsl"

uniform sampler2D colortex1;
uniform float viewWidth;
uniform float viewHeight;

in vec2 coord0;
out vec4 fragColor;

#if SHARPENING == 0
// Off
void SharpenFilter(inout vec3 color, vec2 coord) {}
#elif SHARPENING == 1
// Unsharp Mask
#include "bsl_lib/unsharp_mask.glsl"
#elif SHARPENING == 2
// CAS

#define textureLod0Offset(img, coord, offset) textureLodOffset(img, coord, 0.0f, offset)
#define textureLod0(img, coord) textureLod(img, coord, 0.0f)

void SharpenFilter(inout vec3 color, vec2 textureCoord) {
    // Fetch a 3x3 neighborhood around the pixel 'e'
    vec3 a = textureLod0Offset(colortex1, textureCoord, ivec2(-1,-1)).rgb;
    vec3 b = textureLod0Offset(colortex1, textureCoord, ivec2( 0,-1)).rgb;
    vec3 c = textureLod0Offset(colortex1, textureCoord, ivec2( 1,-1)).rgb;
    vec3 d = textureLod0Offset(colortex1, textureCoord, ivec2(-1, 0)).rgb;
    vec3 e = color;
    vec3 f = textureLod0Offset(colortex1, textureCoord, ivec2( 1, 0)).rgb;
    vec3 g = textureLod0Offset(colortex1, textureCoord, ivec2(-1, 1)).rgb;
    vec3 h = textureLod0Offset(colortex1, textureCoord, ivec2( 0, 1)).rgb;
    vec3 i = textureLod0Offset(colortex1, textureCoord, ivec2( 1, 1)).rgb;

    // Calculate minimum and maximum
    vec3 mnRGB = min(min(min(d, e), min(f, b)), h) + min(min(min(a, g), c), i);
    vec3 mxRGB = max(max(max(d, e), max(f, b)), h) + max(max(max(a, g), c), i);

    // Smooth minimum distance to signal limit divided by smooth max
    vec3 rcpMxRGB = 1.0 / mxRGB;
    vec3 ampRGB = clamp(min(mnRGB, 2.0 - mxRGB) * rcpMxRGB, 0.0, 1.0);

    // Shaping amount of sharpening
    ampRGB = inversesqrt(ampRGB);
    float peak = 8.0 - 3.0 * CAS_AMOUNT;
    vec3 wRGB = -1.0 / (ampRGB * peak);
    vec3 rcpWeightRGB = 1.0 / (1.0 + 4.0 * wRGB);

    // Filter shape
    vec3 window = (b + d) + (f + h);
    vec3 outColor = clamp((window * wRGB + e) * rcpWeightRGB, 0.0, 1.0);
    color = outColor;
}
#endif

void main() {
    vec3 color = texture2DLod(colortex1, coord0, 0).rgb;
    SharpenFilter(color, coord0);
    fragColor = vec4(color, 1.0);
}
